@@ -103,14 +103,13 @@ typedef char s_char;
 /*
  * Poll interval parameters
  */
-#define NTP_UNREACH	24	/* poll unreach threshold */
-#define	NTP_MINPOLL	4	/* log2 min poll interval (16 s) */
+#define NTP_UNREACH	10	/* poll unreach threshold */
+#define	NTP_MINPOLL	3	/* log2 min poll interval (8 s) */
 #define NTP_MINDPOLL	6	/* log2 default min poll (64 s) */
 #define NTP_MAXDPOLL	10	/* log2 default max poll (~17 m) */
 #define	NTP_MAXPOLL	17	/* log2 max poll interval (~36 h) */
-#define NTP_BURST	8	/* packets in burst */
-#define BURST_DELAY	2	/* interburst delay (s) */
-#define	RESP_DELAY	1	/* crypto response delay (s) */
+#define	NTP_RETRY	3	/* max packet retries */
+#define	NTP_MINPKT	1	/* log2 min interburst interval (2 s) */
 
 /*
  * Clock filter algorithm tuning parameters
@@ -126,7 +125,7 @@ typedef char s_char;
 #define	NTP_MAXCLOCK	10	/* max candidates */
 #define	NTP_MAXASSOC	50	/* max associations */
 #define MINDISPERSE	.005	/* min dispersion increment */
-#define MAXDISTANCE	1.	/* max root distance (select threshold) */
+#define MAXDISTANCE	1.5	/* max root distance (select threshold) */
 #define CLOCK_SGATE	3.	/* popcorn spike gate */
 #define HUFFPUFF	900	/* huff-n'-puff sample interval (s) */
 #define MAXHOP		2	/* anti-clockhop threshold */
@@ -221,11 +220,11 @@ struct interface {
 #define TEST2		0x0002	/* bogus packet */
 #define TEST3		0x0004	/* protocol unsynchronized */
 #define TEST4		0x0008	/* access denied */
-#define TEST5		0x0010	/* authentication error */
+#define TEST5		0x0010	/* bad authentication */
 #define TEST6		0x0020	/* bad synch or stratum */
-#define TEST7		0x0040	/* bad header data */
-#define TEST8		0x0080  /* autokey error */
-#define TEST9		0x0100	/* crypto error */
+#define TEST7		0x0040	/* bad header */
+#define TEST8		0x0080  /* bad autokey */
+#define TEST9		0x0100	/* bad crypto */
 #define	PKT_TEST_MASK	(TEST1 | TEST2 | TEST3 | TEST4 | TEST5 |\
 			TEST6 | TEST7 | TEST8 | TEST9)
 /*
@@ -237,14 +236,6 @@ struct interface {
 #define TEST13		0x1000	/* peer unreacable */
 #define	PEER_TEST_MASK	(TEST10 | TEST11 | TEST12 | TEST13)
 
-/*
- * Authentication codes
- */
-#define	AUTH_NONE	0	/* no authentication */
-#define	AUTH_OK		1	/* authentication OK */
-#define	AUTH_ERROR	2	/* authentication error */
-#define	AUTH_CRYPTO	3	/* crypto-NAK */
-
 /*
  * The peer structure. Holds state information relating to the guys
  * we are peering with. Most of this stuff is from section 3.2 of the
@@ -264,7 +255,6 @@ struct peer {
 	u_char	maxpoll;	/* max poll interval */
 	u_int	flags;		/* association flags */
 	u_char	cast_flags;	/* additional flags */
-	u_int	flash;		/* protocol error test tally bits */
 	u_char	last_event;	/* last peer error code */
 	u_char	num_events;	/* number of error events */
 	u_char	ttl;		/* ttl/refclock mode */
@@ -287,8 +277,8 @@ struct peer {
 	u_char	stratum;	/* remote stratum */
 	u_char	ppoll;		/* remote poll interval */
 	s_char	precision;	/* remote clock precision */
-	double	rootdelay;	/* roundtrip delay to primary clock */
-	double	rootdispersion;	/* dispersion to primary clock */
+	double	rootdelay;	/* roundtrip delay to primary source */
+	double	rootdisp;	/* dispersion to primary source */
 	u_int32	refid;		/* remote reference ID */
 	l_fp	reftime;	/* update epoch */
 
@@ -297,22 +287,25 @@ struct peer {
 	 */
 	keyid_t keyid;		/* current key ID */
 #ifdef OPENSSL
-#define clear_to_zero assoc
+#define clear_to_zero opcode
+	u_int32	opcode;		/* last request opcode */
 	associd_t assoc;	/* peer association ID */
 	u_int32	crypto;		/* peer status word */
 	EVP_PKEY *pkey;		/* public key */
 	const EVP_MD *digest;	/* message digest algorithm */
 	char	*subject;	/* certificate subject name */
 	char	*issuer;	/* certificate issuer name */
+	struct cert_info *xinfo; /* issuer certificate */
 	keyid_t	pkeyid;		/* previous key ID */
+	keyid_t	hcookie;	/* host cookie */
 	keyid_t	pcookie;	/* peer cookie */
-	EVP_PKEY *ident_pkey;	/* identity key */
-	tstamp_t fstamp;	/* identity filestamp */
-	BIGNUM	*iffval;	/* IFF/GQ challenge */
-	BIGNUM	*grpkey;	/* GQ group key */
-	struct value cookval;	/* cookie values */
+	const struct pkey_info *ident_pkey; /* identity key */
+	BIGNUM	*iffval;	/* identity challenge (IFF, GQ, MV) */
+	const BIGNUM *grpkey;	/* identity challenge key (GQ) */
+	struct value cookval;	/* receive cookie values */
 	struct value recval;	/* receive autokey values */
 	struct exten *cmmd;	/* extension pointer */
+	u_long	refresh;	/* next refresh epoch */
 
 	/*
 	 * Variables used by authenticated server
@@ -321,7 +314,6 @@ struct peer {
 	int	keynumber;	/* current key number */
 	struct value encrypt;	/* send encrypt values */
 	struct value sndval;	/* send autokey values */
-	struct value tai_leap;	/* send leapsecond table */
 #else /* OPENSSL */
 #define clear_to_zero status
 #endif /* OPENSSL */
@@ -331,9 +323,11 @@ struct peer {
 	 */
 	u_char	status;		/* peer status */
 	u_char	reach;		/* reachability register */
+	int	flash;		/* protocol error test tally bits */
 	u_long	epoch;		/* reference epoch */
-	u_int	burst;		/* packets remaining in burst */
-	u_int	filter_nextpt;	/* index into filter shift register */
+	int	burst;		/* packets remaining in burst */
+	int	retry;		/* retry counter */
+	int	filter_nextpt;	/* index into filter shift register */
 	double	filter_delay[NTP_SHIFT]; /* delay shift register */
 	double	filter_offset[NTP_SHIFT]; /* offset shift register */
 	double	filter_disp[NTP_SHIFT]; /* dispersion shift register */
@@ -346,18 +340,29 @@ struct peer {
 	double	delay;		/* peer roundtrip delay */
 	double	jitter;		/* peer jitter (squares) */
 	double	disp;		/* peer dispersion */
-	double	estbdelay;	/* clock offset to broadcast server */
+
+	/*
+	 * Variables used to correct for packet length and asymmetry.
+	 */
+	double	t21;		/* outbound packet delay */
+	int	t21_bytes;	/* outbound packet length */
+	int	t21_last;	/* last outbound packet length */
+	double	r21;		/* outbound data rate */
+	double	t34;		/* inbound packet delay */
+	int	t34_bytes;	/* inbound packet length */
+	double	r34;		/* inbound data rate */
 
 	/*
 	 * End of clear-to-zero area
 	 */
 	u_long	update;		/* receive epoch */
-	u_int	unreach;	/* unreachable count */
-#define end_clear_to_zero unreach
+#define end_clear_to_zero update
+	int	unreach;	/* watchdog counter */
+	int	throttle;	/* rate control */
 	u_long	outdate;	/* send time last packet */
 	u_long	nextdate;	/* send time next packet */
 	u_long	nextaction;	/* peer local activity timeout (refclocks) */
-	void (*action) P((struct peer *)); /* action timeout function */
+	void (*action) (struct peer *); /* action timeout function */
 
 	/*
 	 * Statistic counters
@@ -368,12 +373,12 @@ struct peer {
 
 	u_long	sent;		/* packets sent */
 	u_long	received;	/* packets received */
-	u_long	processed;	/* packets processed by the protocol */
-	u_long	badauth;	/* packets cryptosum failed */
-	u_long	bogusorg;	/* packets bogus origin */
-	u_long	oldpkt;		/* packets duplicate packet */
-	u_long	seldisptoolarge; /* packets dispersion too large */
-	u_long	selbroken;	/* not used */
+	u_long	processed;	/* packets processed */
+	u_long	badauth;	/* bad authentication (TEST5) */
+	u_long	bogusorg;	/* bogus origin (TEST2, TEST3) */
+	u_long	oldpkt;		/* old duplicate (TEST1) */
+	u_long	seldisptoolarge; /* bad header (TEST6, TEST7) */
+	u_long	selbroken;	/* KoD received */
 };
 
 /*
@@ -419,21 +424,20 @@ struct peer {
  * Values for peer.flags
  */
 #define	FLAG_CONFIG	0x0001	/* association was configured */
-#define	FLAG_AUTHENABLE	0x0002	/* authentication required */
+#define FLAG_PREEMPT	0x0002	/* preemptable association */
 #define	FLAG_AUTHENTIC	0x0004	/* last message was authentic */
-#define FLAG_SKEY	0x0008  /* autokey authentication */
-#define FLAG_MCAST	0x0010  /* multicast client mode */
-#define	FLAG_REFCLOCK	0x0020	/* this is actually a reference clock */
-#define	FLAG_SYSPEER	0x0040	/* this is one of the selected peers */
-#define FLAG_PREFER	0x0080	/* this is the preferred peer */
-#define FLAG_BURST	0x0100	/* burst mode */
-#define FLAG_IBURST	0x0200	/* initial burst mode */
-#define FLAG_NOSELECT	0x0400	/* never select */
-#define FLAG_ASSOC	0x0800	/* autokey request */
-#define FLAG_FIXPOLL	0x1000	/* stick at minpoll */
-#define FLAG_TRUE	0x2000	/* select truechimer */
-#define	FLAG_PREEMPT	0x4000	/* preemptable association */
-#define	FLAG_DYNAMIC	0x8000	/* dynamic addresses - allow configuration even if no interface is found */
+#define	FLAG_REFCLOCK	0x0008	/* this is actually a reference clock */
+#define	FLAG_SYSPEER	0x0010	/* system peer */
+#define FLAG_PREFER	0x0020	/* prefer peer */
+#define FLAG_BURST	0x0040	/* burst mode */
+#define FLAG_FIXPOLL	0x0080	/* stick at minpoll */
+#define FLAG_IBURST	0x0100	/* initial burst mode */
+#define FLAG_NOSELECT	0x0200	/* never select */
+#define FLAG_TRUE	0x0400	/* force truechimer */
+#ifdef OPENSSL
+#define FLAG_SKEY	0x1000  /* autokey authentication */
+#define FLAG_ASSOC	0x2000	/* autokey request */
+#endif /* OPENSSL */
 
 /*
  * Definitions for the clear() routine.  We use memset() to clear
@@ -506,6 +510,12 @@ struct peer {
  /*
  * Macro for sockaddr_storage structures operations
  */
+#ifdef ISC_PLATFORM_HAVESCOPEID
+#define SOCKSCOPE(sock1, sock2) (((struct sockaddr_in6 *)sock1)->sin6_scope_id == ((struct sockaddr_in6 *)sock2)->sin6_scope_id)
+#else
+#define SOCKSCOPE(sock1, sock2) 1
+#endif
+
 #define SOCKCMP(sock1, sock2) \
 	(((struct sockaddr_storage *)sock1)->ss_family \
 	    == ((struct sockaddr_storage *)sock2)->ss_family ? \
@@ -515,7 +525,7 @@ struct peer {
 	    sizeof(struct in_addr)) == 0 : \
 	memcmp(&((struct sockaddr_in6 *)sock1)->sin6_addr, \
 	    &((struct sockaddr_in6 *)sock2)->sin6_addr, \
-	    sizeof(struct in6_addr)) == 0 : \
+	       sizeof(struct in6_addr)) == 0 && SOCKSCOPE(sock1, sock2) : \
 	0)
 
 #define SOCKNUL(sock1) \
@@ -593,14 +603,14 @@ struct peer {
  * and must be converted (except the mac, which isn't, really).
  */
 struct pkt {
-	u_char	li_vn_mode;	/* leap indicator, version and mode */
+	u_char	li_vn_mode;	/* peer leap indicator */
 	u_char	stratum;	/* peer stratum */
 	u_char	ppoll;		/* peer poll interval */
 	s_char	precision;	/* peer clock precision */
-	u_fp	rootdelay;	/* distance to primary clock */
-	u_fp	rootdispersion;	/* clock dispersion */
-	u_int32	refid;		/* reference clock ID */
-	l_fp	reftime;	/* time peer clock was last updated */
+	u_fp	rootdelay;	/* roundtrip delay to primary source */
+	u_fp	rootdisp;	/* dispersion to primary source*/
+	u_int32	refid;		/* reference id */
+	l_fp	reftime;	/* last update time */
 	l_fp	org;		/* originate time stamp */
 	l_fp	rec;		/* receive time stamp */
 	l_fp	xmt;		/* transmit time stamp */
@@ -662,33 +672,46 @@ struct pkt {
  * System event codes
  */
 #define	EVNT_UNSPEC	0	/* unspecified */
-#define	EVNT_SYSRESTART	1	/* system restart */
-#define	EVNT_SYSFAULT	2	/* wsystem or hardware fault */
-#define	EVNT_SYNCCHG	3	/* new leap or synch change */
-#define	EVNT_PEERSTCHG	4	/* new source or stratum */
-#define	EVNT_CLOCKRESET	5	/* clock reset */
-#define	EVNT_BADDATETIM	6	/* invalid time or date */
-#define	EVNT_CLOCKEXCPT	7	/* reference clock exception */
-
+#define	EVNT_NSET	1	/* freq not set */
+#define	EVNT_FSET	2	/* freq set */
+#define	EVNT_SPIK	3	/* spike detect */
+#define	EVNT_FREQ	4	/* freq mode */
+#define	EVNT_SYNC	5	/* clock sync */
+#define	EVNT_SYSRESTART	6	/* restart */
+#define	EVNT_SYSFAULT	7	/* panic stop */
+#define	EVNT_NOPEER	8	/* no sys peer */
+#define	EVNT_ARMED	9	/* leap armed */
+#define	EVNT_DISARMED	10	/* leap disarmed */
+#define	EVNT_LEAP	11	/* leap event */
+#define	EVNT_CLOCKRESET	12	/* clock step */
+#define	EVNT_KERN	13	/* kernel event */
 /*
  * Peer event codes
  */
-#define	EVNT_PEERIPERR	(1 | PEER_EVENT) /* IP error */
-#define	EVNT_PEERAUTH	(2 | PEER_EVENT) /* authentication failure */
-#define	EVNT_UNREACH	(3 | PEER_EVENT) /* change to unreachable */
-#define	EVNT_REACH	(4 | PEER_EVENT) /* change to reachable */
-#define	EVNT_PEERCLOCK	(5 | PEER_EVENT) /* clock exception */
+#define	PEVNT_MOBIL	(1 | PEER_EVENT) /* mobilize */
+#define	PEVNT_DEMOBIL	(2 | PEER_EVENT) /* demobilize */
+#define	PEVNT_UNREACH	(3 | PEER_EVENT) /* unreachable */
+#define	PEVNT_REACH	(4 | PEER_EVENT) /* reachable */
+#define	PEVNT_RESTART	(5 | PEER_EVENT) /* restart */
+#define	PEVNT_REPLY	(6 | PEER_EVENT) /* no reply */
+#define	PEVNT_RATE	(7 | PEER_EVENT) /* rate exceeded */
+#define	PEVNT_DENY	(8 | PEER_EVENT) /* access denied */
+#define PEVNT_ARMED	(9 | PEER_EVENT) /* leap armed */
+#define	PEVNT_NEWPEER	(10 | PEER_EVENT) /* sys peer*/
+#define	PEVNT_CLOCK	(11 | PEER_EVENT) /* clock event */
+#define	PEVNT_AUTH	(12 | PEER_EVENT) /* bad auth */
+#define	PEVNT_POPCORN	(13 | PEER_EVENT) /* popcorn */
 
 /*
  * Clock event codes
  */
 #define	CEVNT_NOMINAL	0	/* unspecified */
-#define	CEVNT_TIMEOUT	1	/* poll timeout */
-#define	CEVNT_BADREPLY	2	/* bad reply format */
-#define	CEVNT_FAULT	3	/* hardware or software fault */
-#define	CEVNT_PROP	4	/* propagation failure */
-#define	CEVNT_BADDATE	5	/* bad date format or value */
-#define	CEVNT_BADTIME	6	/* bad time format or value */
+#define	CEVNT_TIMEOUT	1	/* no reply */
+#define	CEVNT_BADREPLY	2	/* bad format */
+#define	CEVNT_FAULT	3	/* fault */
+#define	CEVNT_PROP	4	/* bad signal */
+#define	CEVNT_BADDATE	5	/* bad date */
+#define	CEVNT_BADTIME	6	/* bad time */
 #define CEVNT_MAX	CEVNT_BADTIME
 
 /*
@@ -705,12 +728,6 @@ struct pkt {
 #define	NTP_HASH_MASK	(NTP_HASH_SIZE-1)
 #define	NTP_HASH_ADDR(src)	sock_hash(src)
 
-/*
- * How we randomize polls.  The poll interval is a power of two. We chose
- * a random interval which is this value plus-minus one second.
- */
-#define RANDPOLL(x)	((1 << (x)) - 1 + (ntp_random() & 0x3))
-
 /*
  * min, min3 and max.  Makes it easier to transliterate the spec without
  * thinking about it.
@@ -764,6 +781,8 @@ struct pkt {
 #define LOOP_HUFFPUFF		9	/* set huff-n'-puff filter length */
 #define LOOP_FREQ		10	/* set initial frequency */
 #define LOOP_KERN_CLEAR		11	/* reset kernel pll parameters */
+#define LOOP_CODEC		12	/* set audio codec frequency */
+#define	LOOP_LEAP		13	/* insert leap after second 23:59 */
 
 /*
  * Configuration items for the stats printer
@@ -771,13 +790,13 @@ struct pkt {
 #define	STATS_FREQ_FILE		1	/* configure drift file */
 #define STATS_STATSDIR		2	/* directory prefix for stats files */
 #define	STATS_PID_FILE		3	/* configure ntpd PID file */
+#define	STATS_LEAP_FILE		4	/* configure ntpd leapseconds file */
 
 #define MJD_1900		15020	/* MJD for 1 Jan 1900 */
 
 /*
  * Default parameters.  We use these in the absence of something better.
  */
-#define	DEFBROADDELAY	4e-3		/* default broadcast offset */
 #define INADDR_NTP	0xe0000101	/* NTP multicast address 224.0.1.1 */
 
 /*
@@ -787,16 +806,17 @@ struct mon_data {
 	struct mon_data *hash_next;	/* next structure in hash list */
 	struct mon_data *mru_next;	/* next structure in MRU list */
 	struct mon_data *mru_prev;	/* previous structure in MRU list */
-	u_long drop_count;		/* dropped due RESLIMIT*/
-	double avg_interval;		/* average interpacket interval */
-	u_long lasttime;		/* interval since last packet */
-	u_long count;			/* total packet count */
+	int	flags;			/* restrict flags */
+	int	leak;			/* leaky bucket accumulator */
+	int	count;			/* total packet count */
+	u_long	firsttime;		/* first time found */
+	u_long	lasttime;		/* last time found */
 	struct sockaddr_storage rmtadr;	/* address of remote host */
 	struct interface *interface;	/* interface on which this arrived */
-	u_short rmtport;		/* remote port last came from */
-	u_char mode;			/* mode of incoming packet */
-	u_char version;			/* version of incoming packet */
-	u_char cast_flags;		/* flags MDF_?CAST */
+	u_short	rmtport;		/* remote port last came from */
+	u_char	mode;			/* packet mode */
+	u_char	version;		/* packet version */
+	u_char	cast_flags;		/* flags MDF_?CAST */
 };
 
 /*
@@ -857,19 +877,19 @@ struct restrictlist6 {
 #define	RES_NOTRAP		0x100	/* mode 6/7 set trap denied */
 #define	RES_LPTRAP		0x200	/* mode 6/7 low priority trap */
 
-#define RES_DEMOBILIZE		0x400	/* send kiss of death packet */
+#define RES_KOD			0x400	/* send kiss of death packet */
 #define RES_TIMEOUT		0x800	/* timeout this entry */
 
 #define	RES_ALLFLAGS		(RES_FLAGS | RES_NOQUERY |\
 				    RES_NOMODIFY | RES_NOTRAP |\
-				    RES_LPTRAP | RES_DEMOBILIZE |\
+				    RES_LPTRAP | RES_KOD |\
 				    RES_TIMEOUT)
 
 /*
  * Match flags
  */
-#define	RESM_INTERFACE		0x1	/* this is an interface */
-#define	RESM_NTPONLY		0x2	/* match ntp port only */
+#define	RESM_INTERFACE		0x1000	/* this is an interface */
+#define	RESM_NTPONLY		0x2000	/* match ntp port only */
 
 /*
  * Restriction configuration ops
