@@ -47,8 +47,8 @@ struct req_proc {
 	short needs_auth;	/* true when authentication needed */
 	short sizeofitem;	/* size of request data item (older size)*/
 	short v6_sizeofitem;	/* size of request data item (new size)*/
-	void (*handler) P((struct sockaddr_storage *, struct interface *,
-			   struct req_pkt *));	/* routine to handle request */
+	void (*handler) (struct sockaddr_storage *, struct interface *,
+			   struct req_pkt *);	/* routine to handle request */
 };
 
 /*
@@ -58,58 +58,58 @@ static	struct req_proc univ_codes[] = {
 	{ NO_REQUEST,		NOAUTH,	 0,	0 }
 };
 
-static	void	req_ack	P((struct sockaddr_storage *, struct interface *, struct req_pkt *, int));
-static	char *	prepare_pkt	P((struct sockaddr_storage *, struct interface *, struct req_pkt *, u_int));
-static	char *	more_pkt	P((void));
-static	void	flush_pkt	P((void));
-static	void	peer_list	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	peer_list_sum	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	peer_info	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	peer_stats	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	sys_info	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	sys_stats	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	mem_stats	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	io_stats	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	timer_stats	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	loop_info	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_conf		P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_unconf	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	set_sys_flag	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	clr_sys_flag	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	setclr_flags	P((struct sockaddr_storage *, struct interface *, struct req_pkt *, u_long));
-static	void	list_restrict	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_resaddflags	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_ressubflags	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_unrestrict	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_restrict	P((struct sockaddr_storage *, struct interface *, struct req_pkt *, int));
-static	void	mon_getlist_0	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	mon_getlist_1	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	reset_stats	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	reset_peer	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_key_reread	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	trust_key	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	untrust_key	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_trustkey	P((struct sockaddr_storage *, struct interface *, struct req_pkt *, u_long));
-static	void	get_auth_info	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	reset_auth_stats P((void));
-static	void	req_get_traps	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	req_set_trap	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	req_clr_trap	P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_setclr_trap	P((struct sockaddr_storage *, struct interface *, struct req_pkt *, int));
-static	void	set_request_keyid P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	set_control_keyid P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	get_ctl_stats   P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	get_if_stats    P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	do_if_reload    P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
+static	void	req_ack	(struct sockaddr_storage *, struct interface *, struct req_pkt *, int);
+static	char *	prepare_pkt	(struct sockaddr_storage *, struct interface *, struct req_pkt *, u_int);
+static	char *	more_pkt	(void);
+static	void	flush_pkt	(void);
+static	void	peer_list	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	peer_list_sum	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	peer_info	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	peer_stats	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	sys_info	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	sys_stats	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	mem_stats	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	io_stats	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	timer_stats	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	loop_info	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_conf		(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_unconf	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	set_sys_flag	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	clr_sys_flag	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	setclr_flags	(struct sockaddr_storage *, struct interface *, struct req_pkt *, u_long);
+static	void	list_restrict	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_resaddflags	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_ressubflags	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_unrestrict	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_restrict	(struct sockaddr_storage *, struct interface *, struct req_pkt *, int);
+static	void	mon_getlist_0	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	mon_getlist_1	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	reset_stats	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	reset_peer	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_key_reread	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	trust_key	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	untrust_key	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_trustkey	(struct sockaddr_storage *, struct interface *, struct req_pkt *, u_long);
+static	void	get_auth_info	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	reset_auth_stats (void);
+static	void	req_get_traps	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	req_set_trap	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	req_clr_trap	(struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_setclr_trap	(struct sockaddr_storage *, struct interface *, struct req_pkt *, int);
+static	void	set_request_keyid (struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	set_control_keyid (struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	get_ctl_stats   (struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	get_if_stats    (struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	do_if_reload    (struct sockaddr_storage *, struct interface *, struct req_pkt *);
 #ifdef KERNEL_PLL
-static	void	get_kernel_info P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
+static	void	get_kernel_info (struct sockaddr_storage *, struct interface *, struct req_pkt *);
 #endif /* KERNEL_PLL */
 #ifdef REFCLOCK
-static	void	get_clock_info P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
-static	void	set_clock_fudge P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
+static	void	get_clock_info (struct sockaddr_storage *, struct interface *, struct req_pkt *);
+static	void	set_clock_fudge (struct sockaddr_storage *, struct interface *, struct req_pkt *);
 #endif	/* REFCLOCK */
 #ifdef REFCLOCK
-static	void	get_clkbug_info P((struct sockaddr_storage *, struct interface *, struct req_pkt *));
+static	void	get_clkbug_info (struct sockaddr_storage *, struct interface *, struct req_pkt *);
 #endif	/* REFCLOCK */
 
 /*
@@ -400,6 +400,15 @@ flush_pkt(void)
 
 
 
+/*
+ * Given a buffer, return the packet mode
+ */
+int
+get_packet_mode(struct recvbuf *rbufp)
+{
+	struct req_pkt *inpkt = (struct req_pkt *)&rbufp->recv_pkt;
+	return (INFO_MODE(inpkt->rm_vn_mode));
+}
 /*
  * process_private - process private mode (7) packets
  */
@@ -409,7 +418,6 @@ process_private(
 	int mod_okay
 	)
 {
-	static u_long quiet_until;
 	struct req_pkt *inpkt;
 	struct req_pkt_tail *tailinpkt;
 	struct sockaddr_storage *srcadr;
@@ -445,14 +453,8 @@ process_private(
 	    || (++ec, INFO_MBZ(inpkt->mbz_itemsize) != 0)
 	    || (++ec, rbufp->recv_length < REQ_LEN_HDR)
 		) {
-		NLOG(NLOG_SYSEVENT)
-			if (current_time >= quiet_until) {
-				msyslog(LOG_ERR,
-					"process_private: drop test %d"
-					" failed, pkt from %s",
-					ec, stoa(srcadr));
-				quiet_until = current_time + 60;
-			}
+		msyslog(LOG_ERR, "process_private: INFO_ERR_FMT: test %d failed, pkt from %s", ec, stoa(srcadr));
+		req_ack(srcadr, inter, inpkt, INFO_ERR_FMT);
 		return;
 	}
 
@@ -503,10 +505,10 @@ process_private(
 	 */
 	temp_size = INFO_ITEMSIZE(inpkt->mbz_itemsize);
 	if ((temp_size != proc->sizeofitem &&
-	     temp_size != proc->v6_sizeofitem) &&
+	    temp_size != proc->v6_sizeofitem) &&
 	    !(inpkt->implementation == IMPL_XNTPD &&
-	      inpkt->request == REQ_CONFIG &&
-	      temp_size == sizeof(struct old_conf_peer))) {
+	    inpkt->request == REQ_CONFIG &&
+	    temp_size == sizeof(struct old_conf_peer))) {
 #ifdef DEBUG
 		if (debug > 2)
 			printf("process_private: wrong item size, received %d, should be %d or %d\n",
@@ -786,8 +788,6 @@ peer_list_sum(
 				    ips->flags |= INFO_FLAG_CONFIG;
 				if (pp->flags & FLAG_REFCLOCK)
 				    ips->flags |= INFO_FLAG_REFCLOCK;
-				if (pp->flags & FLAG_AUTHENABLE)
-				    ips->flags |= INFO_FLAG_AUTHENABLE;
 				if (pp->flags & FLAG_PREFER)
 				    ips->flags |= INFO_FLAG_PREFER;
 				if (pp->flags & FLAG_BURST)
@@ -887,8 +887,6 @@ peer_info (
 		    ip->flags |= INFO_FLAG_CONFIG;
 		if (pp->flags & FLAG_REFCLOCK)
 		    ip->flags |= INFO_FLAG_REFCLOCK;
-		if (pp->flags & FLAG_AUTHENABLE)
-		    ip->flags |= INFO_FLAG_AUTHENABLE;
 		if (pp->flags & FLAG_PREFER)
 		    ip->flags |= INFO_FLAG_PREFER;
 		if (pp->flags & FLAG_BURST)
@@ -909,11 +907,11 @@ peer_info (
 		ip->unreach = (u_char) pp->unreach;
 		ip->flash = (u_char)pp->flash;
 		ip->flash2 = (u_short) pp->flash;
-		ip->estbdelay = HTONS_FP(DTOFP(pp->estbdelay));
+		ip->estbdelay = HTONS_FP(DTOFP(pp->delay));
 		ip->ttl = pp->ttl;
 		ip->associd = htons(pp->associd);
 		ip->rootdelay = HTONS_FP(DTOUFP(pp->rootdelay));
-		ip->rootdispersion = HTONS_FP(DTOUFP(pp->rootdispersion));
+		ip->rootdispersion = HTONS_FP(DTOUFP(pp->rootdisp));
 		ip->refid = pp->refid;
 		HTONL_FP(&pp->reftime, &ip->reftime);
 		HTONL_FP(&pp->org, &ip->org);
@@ -1029,8 +1027,6 @@ peer_stats (
 		    ip->flags |= INFO_FLAG_CONFIG;
 		if (pp->flags & FLAG_REFCLOCK)
 		    ip->flags |= INFO_FLAG_REFCLOCK;
-		if (pp->flags & FLAG_AUTHENABLE)
-		    ip->flags |= INFO_FLAG_AUTHENABLE;
 		if (pp->flags & FLAG_PREFER)
 		    ip->flags |= INFO_FLAG_PREFER;
 		if (pp->flags & FLAG_BURST)
@@ -1096,7 +1092,7 @@ sys_info(
 	is->stratum = sys_stratum;
 	is->precision = sys_precision;
 	is->rootdelay = htonl(DTOFP(sys_rootdelay));
-	is->rootdispersion = htonl(DTOUFP(sys_rootdispersion));
+	is->rootdispersion = htonl(DTOUFP(sys_rootdisp));
 	is->frequency = htonl(DTOFP(sys_jitter));
 	is->stability = htonl(DTOUFP(clock_stability));
 	is->refid = sys_refid;
@@ -1151,9 +1147,9 @@ sys_stats(
 	ss->timeup = htonl((u_int32)current_time);
 	ss->timereset = htonl((u_int32)(current_time - sys_stattime));
 	ss->denied = htonl((u_int32)sys_restricted);
-	ss->oldversionpkt = htonl((u_int32)sys_oldversionpkt);
-	ss->newversionpkt = htonl((u_int32)sys_newversionpkt);
-	ss->unknownversion = htonl((u_int32)sys_unknownversion);
+	ss->oldversionpkt = htonl((u_int32)sys_oldversion);
+	ss->newversionpkt = htonl((u_int32)sys_newversion);
+	ss->unknownversion = htonl((u_int32)sys_declined);
 	ss->badlength = htonl((u_int32)sys_badlength);
 	ss->processed = htonl((u_int32)sys_processed);
 	ss->badauth = htonl((u_int32)sys_badauth);
@@ -1299,7 +1295,7 @@ loop_info(
 	extern double last_offset;
 	extern double drift_comp;
 	extern int tc_counter;
-	extern u_long sys_clocktime;
+	extern u_long sys_epoch;
 
 	li = (struct info_loop *)prepare_pkt(srcadr, inter, inpkt,
 	    sizeof(struct info_loop));
@@ -1309,7 +1305,7 @@ loop_info(
 	DTOLFP(drift_comp * 1e6, &ltmp);
 	HTONL_FP(&ltmp, &li->drift_comp);
 	li->compliance = htonl((u_int32)(tc_counter));
-	li->watchdog_timer = htonl((u_int32)(current_time - sys_clocktime));
+	li->watchdog_timer = htonl((u_int32)(current_time - sys_epoch));
 
 	(void) more_pkt();
 	flush_pkt();
@@ -1326,7 +1322,6 @@ do_conf(
 	struct req_pkt *inpkt
 	)
 {
-	static u_long soonest_ifrescan_time = 0;
 	int items;
 	u_int fl;
 	struct conf_peer *cp; 
@@ -1352,9 +1347,9 @@ do_conf(
 		    && temp_cp.hmode != MODE_CLIENT
 		    && temp_cp.hmode != MODE_BROADCAST)
 		    fl = 1;
-		if (temp_cp.flags & ~(CONF_FLAG_AUTHENABLE | CONF_FLAG_PREFER
-				  | CONF_FLAG_BURST | CONF_FLAG_IBURST | CONF_FLAG_SKEY))
-		    fl = 1;
+		if (temp_cp.flags & ~(CONF_FLAG_PREFER | CONF_FLAG_BURST |
+		    CONF_FLAG_IBURST | CONF_FLAG_SKEY))
+			fl = 1;
 		cp = (struct conf_peer *)
 		    ((char *)cp + INFO_ITEMSIZE(inpkt->mbz_itemsize));
 	}
@@ -1376,8 +1371,6 @@ do_conf(
 		memset((char *)&peeraddr, 0, sizeof(struct sockaddr_storage));
 
 		fl = 0;
-		if (temp_cp.flags & CONF_FLAG_AUTHENABLE)
-			fl |= FLAG_AUTHENABLE;
 		if (temp_cp.flags & CONF_FLAG_PREFER)
 			fl |= FLAG_PREFER;
 		if (temp_cp.flags & CONF_FLAG_BURST)
@@ -1420,23 +1413,6 @@ do_conf(
 			req_ack(srcadr, inter, inpkt, INFO_ERR_NODATA);
 			return;
 		}
-
-		/*
-		 * ntp_intres.c uses REQ_CONFIG/doconf() to add each
-		 * server after its name is resolved.  If we have been
-		 * disconnected from the network, it may notice the
-		 * network has returned and add the first server while
-		 * the relevant interface is still disabled, awaiting
-		 * the next interface rescan.  To get things moving
-		 * more quickly, trigger an interface scan now, except
-		 * if we have done so in the last half minute.
-		 */
-		if (soonest_ifrescan_time < current_time) {
-			soonest_ifrescan_time = current_time + 30;
-			timer_interfacetimeout(current_time);
-			DPRINTF(1, ("do_conf triggering interface rescan\n"));
-		}
-
 		cp = (struct conf_peer *)
 		    ((char *)cp + INFO_ITEMSIZE(inpkt->mbz_itemsize));
 	}
@@ -1927,9 +1903,10 @@ mon_getlist_0(
 	    v6sizeof(struct info_monitor));
 	for (md = mon_mru_list.mru_next; md != &mon_mru_list && im != 0;
 	     md = md->mru_next) {
-		im->lasttime = htonl((u_int32)md->avg_interval);
+		im->lasttime = htonl((u_int32)((current_time -
+		    md->firsttime) / md->count));
 		im->firsttime = htonl((u_int32)(current_time - md->lasttime));
-		im->lastdrop = htonl((u_int32)md->drop_count);
+		im->lastdrop = htonl((u_int32)md->flags);
 		im->count = htonl((u_int32)(md->count));
 		if (md->rmtadr.ss_family == AF_INET6) {
 			if (!client_v6_capable)
@@ -1972,9 +1949,10 @@ mon_getlist_1(
 	    v6sizeof(struct info_monitor_1));
 	for (md = mon_mru_list.mru_next; md != &mon_mru_list && im != 0;
 	     md = md->mru_next) {
-		im->lasttime = htonl((u_int32)md->avg_interval);
+		im->lasttime = htonl((u_int32)((current_time -
+		    md->firsttime) / md->count));
 		im->firsttime = htonl((u_int32)(current_time - md->lasttime));
-		im->lastdrop = htonl((u_int32)md->drop_count);
+		im->lastdrop = htonl((u_int32)md->flags);
 		im->count = htonl((u_int32)md->count);
 		if (md->rmtadr.ss_family == AF_INET6) {
 			if (!client_v6_capable)
@@ -2008,7 +1986,7 @@ mon_getlist_1(
  */
 struct reset_entry {
 	int flag;		/* flag this corresponds to */
-	void (*handler) P((void)); /* routine to handle request */
+	void (*handler) (void); /* routine to handle request */
 };
 
 struct reset_entry reset_entries[] = {
@@ -2699,6 +2677,7 @@ set_clock_fudge(
 
 	memset((char *)&addr, 0, sizeof addr);
 	memset((char *)&clock_stat, 0, sizeof clock_stat);
+	memset((char *)&tmp_clock, 0, sizeof tmp_clock);
 	items = INFO_NITEMS(inpkt->err_nitems);
 	cf = (struct conf_fudge *) inpkt->data;
 
