@@ -23,6 +23,10 @@
 # include "ntp_timer.h"
 #endif
 
+#ifdef KERNEL_PLL
+#include "ntp_syscall.h"
+#endif /* KERNEL_PLL */
+
 /*
  * These routines provide support for the event timer.	The timer is
  * implemented by an interrupt routine which sets a flag once every
@@ -33,22 +37,26 @@
  * dispatched to the transmit procedure.  Finally, we call the hourly
  * procedure to do cleanup and print a message.
  */
-
+volatile int interface_interval = 300;     /* update interface every 5 minutes as default */
+	  
 /*
- * Alarm flag.	The mainline code imports this.
+ * Alarm flag. The mainline code imports this.
  */
 volatile int alarm_flag;
 
 /*
- * The counters
+ * The counters and timeouts
  */
-static	u_long adjust_timer;		/* second timer */
-static	u_long keys_timer;		/* minute timer */
-static	u_long hourly_timer;		/* hour timer */
-static	u_long huffpuff_timer;		/* huff-n'-puff timer */
+static  u_long interface_timer;	/* interface update timer */
+static	u_long adjust_timer;	/* second timer */
+static	u_long stats_timer;	/* stats timer */
+static	u_long huffpuff_timer;	/* huff-n'-puff timer */
+u_long	leapsec;		/* leapseconds countdown */
 #ifdef OPENSSL
-static	u_long revoke_timer;		/* keys revoke timer */
-u_char	sys_revoke = KEY_REVOKE;	/* keys revoke timeout (log2 s) */
+static	u_long revoke_timer;	/* keys revoke timer */
+static	u_long keys_timer;	/* session key timer */
+u_long	sys_revoke = KEY_REVOKE; /* keys revoke timeout (log2 s) */
+u_long	sys_automax = NTP_AUTOMAX; /* key list timeout (log2 s) */
 #endif /* OPENSSL */
 
 /*
@@ -57,9 +65,10 @@ u_char	sys_revoke = KEY_REVOKE;	/* keys revoke timeout (log2 s) */
 volatile u_long alarm_overflow;
 
 #define MINUTE	60
-#define HOUR	(60*60)
+#define HOUR	(60 * MINUTE)
+#define	DAY	(24 * HOUR)
 
-u_long current_time;
+u_long current_time;		/* seconds since startup */
 
 /*
  * Stats.  Number of overflows and number of calls to transmit().
@@ -76,9 +85,60 @@ static int vmsinc[2];		/* timer increment */
 #if defined SYS_WINNT
 static HANDLE WaitableTimerHandle = NULL;
 #else
-static	RETSIGTYPE alarming P((int));
+static	RETSIGTYPE alarming (int);
 #endif /* SYS_WINNT */
 
+#if !defined(VMS)
+# if !defined SYS_WINNT || defined(SYS_CYGWIN32)
+#  ifndef HAVE_TIMER_SETTIME
+	struct itimerval itimer;
+#  else 
+	static timer_t ntpd_timerid;
+	struct itimerspec itimer;
+#  endif /* HAVE_TIMER_SETTIME */
+# endif /* SYS_WINNT */
+#endif /* VMS */
+
+/*
+ * reinit_timer - reinitialize interval timer.
+ */
+void 
+reinit_timer(void)
+{
+#if !defined(SYS_WINNT) && !defined(VMS)
+#  if defined(HAVE_TIMER_CREATE) && defined(HAVE_TIMER_SETTIME)
+	timer_gettime(ntpd_timerid, &itimer);
+	if (itimer.it_value.tv_sec < 0 || itimer.it_value.tv_sec > (1<<EVENT_TIMEOUT)) {
+		itimer.it_value.tv_sec = (1<<EVENT_TIMEOUT);
+	}
+	if (itimer.it_value.tv_nsec < 0 ) {
+		itimer.it_value.tv_nsec = 0;
+	}
+	if (itimer.it_value.tv_sec == 0 && itimer.it_value.tv_nsec == 0) {
+		itimer.it_value.tv_sec = (1<<EVENT_TIMEOUT);
+		itimer.it_value.tv_nsec = 0;
+	}
+	itimer.it_interval.tv_sec = (1<<EVENT_TIMEOUT);
+	itimer.it_interval.tv_nsec = 0;
+	timer_settime(ntpd_timerid, 0 /*!TIMER_ABSTIME*/, &itimer, NULL);
+#  else
+	getitimer(ITIMER_REAL, &itimer);
+	if (itimer.it_value.tv_sec < 0 || itimer.it_value.tv_sec > (1<<EVENT_TIMEOUT)) {
+		itimer.it_value.tv_sec = (1<<EVENT_TIMEOUT);
+	}
+	if (itimer.it_value.tv_usec < 0 ) {
+		itimer.it_value.tv_usec = 0;
+	}
+	if (itimer.it_value.tv_sec == 0 && itimer.it_value.tv_usec == 0) {
+		itimer.it_value.tv_sec = (1<<EVENT_TIMEOUT);
+		itimer.it_value.tv_usec = 0;
+	}
+	itimer.it_interval.tv_sec = (1<<EVENT_TIMEOUT);
+	itimer.it_interval.tv_usec = 0;
+	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);
+#  endif
+# endif /* VMS */
+}
 
 /*
  * init_timer - initialize the timer data structures
@@ -86,20 +146,10 @@ static	RETSIGTYPE alarming P((int));
 void
 init_timer(void)
 {
-#if !defined(VMS)
-# if !defined SYS_WINNT || defined(SYS_CYGWIN32)
-#  ifndef HAVE_TIMER_SETTIME
-	struct itimerval itimer;
-#  else
-	static timer_t ntpd_timerid;	/* should be global if we ever want */
-					/* to kill timer without rebooting ... */
-	struct itimerspec itimer;
-#  endif /* HAVE_TIMER_SETTIME */
-# else /* SYS_WINNT */
-	HANDLE hToken;
+# if defined SYS_WINNT & !defined(SYS_CYGWIN32)
+	HANDLE hToken = INVALID_HANDLE_VALUE;
 	TOKEN_PRIVILEGES tkp;
 # endif /* SYS_WINNT */
-#endif /* !VMS */
 
 	/*
 	 * Initialize...
@@ -107,8 +157,9 @@ init_timer(void)
 	alarm_flag = 0;
 	alarm_overflow = 0;
 	adjust_timer = 1;
-	hourly_timer = HOUR;
+	stats_timer = 0;
 	huffpuff_timer = 0;
+	interface_timer = 0;
 	current_time = 0;
 	timer_overflows = 0;
 	timer_xmtcalls = 0;
@@ -208,38 +259,55 @@ get_timer_handle(void)
 #endif
 
 /*
- * timer - dispatch anyone who needs to be
+ * timer - event timer
  */
 void
 timer(void)
 {
 	register struct peer *peer, *next_peer;
-#ifdef OPENSSL
-	u_char	statstr[NTP_MAXSTRLEN]; /* statistics for filegen */
-#endif /* OPENSSL */
-	u_int n;
-
-	current_time += (1<<EVENT_TIMEOUT);
+	u_int	n;
 
 	/*
-	 * Adjustment timeout first.
+	 * The basic timerevent is one second. This is used to adjust
+	 * the system clock in time and frequency, implement the
+	 * kiss-o'-deatch function and implement the association
+	 * polling function..
 	 */
+	current_time += (1<<EVENT_TIMEOUT);
 	if (adjust_timer <= current_time) {
 		adjust_timer += 1;
 		adj_host_clock();
-		kod_proto();
+#ifdef REFCLOCK
+		for (n = 0; n < NTP_HASH_SIZE; n++) {
+			for (peer = peer_hash[n]; peer != 0; peer = next_peer) {
+				next_peer = peer->next;
+				if (peer->flags & FLAG_REFCLOCK)
+					refclock_timer(peer);
+			}
+		}
+#endif /* REFCLOCK */
 	}
 
 	/*
-	 * Now dispatch any peers whose event timer has expired. Be careful
-	 * here, since the peer structure might go away as the result of
-	 * the call.
+	 * Now dispatch any peers whose event timer has expired. Be
+	 * careful here, since the peer structure might go away as the
+	 * result of the call.
 	 */
-	for (n = 0; n < HASH_SIZE; n++) {
+	for (n = 0; n < NTP_HASH_SIZE; n++) {
 		for (peer = peer_hash[n]; peer != 0; peer = next_peer) {
 			next_peer = peer->next;
-			if (peer->action && peer->nextaction <= current_time)
+			if (peer->action && peer->nextaction <=
+			    current_time)
 	  			peer->action(peer);
+
+			/*
+			 * Restrain the non-burst packet rate not more
+			 * than one packet every 16 seconds. This is
+			 * usually tripped using iburst and minpoll of
+			 * 128 s or less.
+			 */
+			if (peer->throttle > 0)
+				peer->throttle--;
 			if (peer->nextdate <= current_time) {
 #ifdef REFCLOCK
 				if (peer->flags & FLAG_REFCLOCK)
@@ -254,15 +322,53 @@ timer(void)
 	}
 
 	/*
-	 * Garbage collect expired keys.
+	 * Orphan mode is active when enabled and when no servers less
+	 * than the orphan statum are available. A server with no other
+	 * synchronization source is an orphan It shows offset zero and
+	 * reference ID the loopback address.
 	 */
-	if (keys_timer <= current_time) {
-		keys_timer += MINUTE;
-		auth_agekeys();
+	if (sys_orphan < STRATUM_UNSPEC && sys_peer == NULL) {
+		if (sys_leap == LEAP_NOTINSYNC)
+			sys_leap = LEAP_NOWARNING;
+		sys_stratum = sys_orphan;
+		if (sys_stratum > 1)
+			sys_refid = htonl(LOOPBACKADR);
+		else
+			memcpy(&sys_refid, "LOOP", 4);
+		sys_rootdelay = 0;
+		sys_rootdisp = sys_mindisp;
 	}
 
 	/*
-	 * Huff-n'-puff filter
+	 * Leapseconds. If a leap is pending, decrement the time
+	 * remaining. If less than one day remains, set the leap bits.
+	 * When no time remains, clear the leap bits and increment the
+	 * TAI. If kernel suppport is not available, do the leap
+	 * crudely. Note a leap cannot be pending unless the clock is
+	 * set.
+	 */
+	if (leapsec > 0) {
+		leapsec--;
+		if (leap_sec == 0) {
+			sys_leap = LEAP_NOWARNING;
+			sys_tai = leap_tai;
+#ifdef KERNEL_PLL
+			if (!(pll_control && kern_enable))
+				step_systime(-1.0);
+#else /* KERNEL_PLL */
+			step_systime(-1.0);
+#endif /* KERNEL_PLL */
+			report_event(EVNT_LEAP, NULL, NULL);
+		} else {
+			if (leapsec < DAY)
+				sys_leap = LEAP_ADDSECOND;
+			if (leap_tai > 0)
+				sys_tai = leap_tai - 1;
+		}
+	}
+
+	/*
+	 * Update huff-n'-puff filter.
 	 */
 	if (huffpuff_timer <= current_time) {
 		huffpuff_timer += HUFFPUFF;
@@ -270,27 +376,46 @@ timer(void)
 	}
 
 #ifdef OPENSSL
+
 	/*
-	 * Garbage collect old keys and generate new private value
+	 * Garbage collect expired keys.
 	 */
-	if (revoke_timer <= current_time) {
-		revoke_timer += RANDPOLL(sys_revoke);
-		expire_all();
-		sprintf(statstr, "refresh ts %u", ntohl(hostval.tstamp));
-		record_crypto_stats(NULL, statstr);
-#ifdef DEBUG
-		if (debug)
-			printf("timer: %s\n", statstr);
-#endif
+	if (keys_timer <= current_time) {
+		keys_timer += 1 << sys_automax;
+		auth_agekeys();
+	}
+
+	/*
+	 * Garbage collect key list and generate new private value. The
+	 * timer runs only after initial synchronization and fires about
+	 * once per day.
+	 */
+	if (revoke_timer <= current_time && sys_leap !=
+	    LEAP_NOTINSYNC) {
+		revoke_timer += 1 << sys_revoke;
+		RAND_bytes((u_char *)&sys_private, 4);
 	}
 #endif /* OPENSSL */
 
 	/*
-	 * Finally, call the hourly routine.
+	 * Interface update timer
 	 */
-	if (hourly_timer <= current_time) {
-		hourly_timer += HOUR;
-		hourly_stats();
+	if (interface_interval && interface_timer <= current_time) {
+		timer_interfacetimeout(current_time +
+		    interface_interval);
+#ifdef DEBUG
+	  	if (debug > 1)
+	 	 	printf("timer: interface update\n");
+#endif
+		interface_update(NULL, NULL);
+	}
+	
+	/*
+	 * Finally, write hourly stats.
+	 */
+	if (stats_timer <= current_time) {
+		stats_timer += HOUR;
+		write_stats();
 	}
 }
 
@@ -322,6 +447,12 @@ alarming(
 }
 #endif /* SYS_WINNT */
 
+void
+timer_interfacetimeout(u_long timeout)
+{
+	interface_timer = timeout;
+}
+
 
 /*
  * timer_clr_stats - clear timer module stat counters
