@@ -1,21 +1,21 @@
-repotype=stable
+repotype=dev
 # version=Protocol.Major.Minor
 # odd minor numbers are for -dev, even minor numbers are for -stable
 proto=4
 major=2
-minor=4
+minor=5
 version=${proto}.${major}.${minor}
 # Point.  3 cases:
 # - Numeric values increment
 # - empty 'increments' to 1
 # - NEW 'increments' to empty
-point=NEW
+point=113
 # Special.  Normally unused.  A suffix.
 #special=ag
 special=
 # ReleaseCandidate. 'yes' or 'no'.
+#releasecandidate=yes
 releasecandidate=no
-#releasecandidate=no
 # ChangeLog tag
 CLTAG=NTP_4_2_0
 ###
@@ -25,5 +25,4 @@ CLTAG=NTP_4_2_0
 # - - rcpoint gets set to 0
 # - - releasecandidate gets set to no
 # - GRONK is for -dev
-rcpoint=
-#rcpoint=GRONK
+rcpoint=GRONK
